@@ -7,6 +7,7 @@ from __future__ import annotations
 
 __author__ = "Luis Barroso-Luque, Fengyu Xie"
 
+import warnings
 from abc import ABCMeta, abstractmethod
 from collections.abc import Sequence
 from numbers import Integral
@@ -51,7 +52,9 @@ class CVXCanonicals(NamedTuple):
             SimpleNamespace with auxiliary cp.Variable or cp.Expression objects.
             The namespace should be defined by the Regressor generating it.
         constraints (list of cp.Constaint):
-            List of constraints.
+            List of constraints intrinsic to regression problem.
+        user_constraints (list of cp.Constaint):
+            List of user-defined constraints.
     """
 
     problem: cp.Problem
@@ -59,15 +62,16 @@ class CVXCanonicals(NamedTuple):
     beta: cp.Variable
     parameters: SimpleNamespace | None
     auxiliaries: SimpleNamespace | None
-    constraints: list[cp.Expression] | None
+    constraints: list[cp.Expression | cp.Constraint] | None
+    user_constraints: list[cp.Expression | cp.Constraint] | None
 
 
 class CVXRegressor(RegressorMixin, LinearModel, metaclass=ABCMeta):
     r"""Abstract base class for Regressors using cvxpy with a sklearn interface.
 
     Note cvxpy can use one of many 3rd party solvers, default is most often
-    CVXOPT or ECOS. For integer and mixed integer problems options include SCIP (open source)
-    and Gurobi, among other commercial solvers.
+    CVXOPT or ECOS. For integer and mixed integer problems options include
+    SCIP (open source) and Gurobi, among other commercial solvers.
 
     The solver can be specified by setting the solver keyword argument.
     And can solver specific settings can be set by passing a dictionary of
@@ -95,14 +99,16 @@ class CVXRegressor(RegressorMixin, LinearModel, metaclass=ABCMeta):
 
     Attributes:
         coef_ (NDArray):
-            Parameter vector (:math:`\beta` in the cost function formula) of shape (n_features,).
+            Parameter vector (:math:`\beta` in the cost function formula) of shape
+            (n_features,).
         intercept_ (float):
             Independent term in decision function.
         canonicals_ (SimpleNamespace):
             Namespace that contains underlying cvxpy objects used to define
             the optimization problem. The objects included are the following:
                 - objective - the objective function.
-                - beta - variable to be optimized (corresponds to the estimated coef_ attribute).
+                - beta - variable to be optimized (corresponds to the estimated
+                         coef_ attribute).
                 - parameters - hyper-parameters
                 - auxiliaries - auxiliary variables and expressions
                 - constraints - solution constraints
@@ -143,8 +149,9 @@ class CVXRegressor(RegressorMixin, LinearModel, metaclass=ABCMeta):
     ):
         """Fit the linear model coefficients.
 
-        Prepares the  fit data input, generates cvxpy objects to represent the minimization
-        objective, and solves the regression problem using the given solver.
+        Prepares the  fit data input, generates cvxpy objects to represent the
+        minimization objective, and solves the regression problem using the given
+        solver.
 
         Args:
             X (ArrayLike):
@@ -167,6 +174,40 @@ class CVXRegressor(RegressorMixin, LinearModel, metaclass=ABCMeta):
             X, y, accept_sparse=False, y_numeric=True, multi_output=False
         )
 
+        X, y, X_offset, y_offset, X_scale = self._preprocess_data(X, y, sample_weight)
+
+        self._validate_params(X, y)
+
+        # TODO test theses cases
+        if not hasattr(self, "canonicals_"):
+            self.generate_problem(X, y, preprocess_data=False)
+        elif not np.array_equal(self.cached_X_, X) or not np.array_equal(
+            self.cached_y_, y
+        ):
+            if self.canonicals_.user_constraints:
+                warnings.warn(
+                    "User constraints are set on a problem with different data (X, y). "
+                    "These constraints will be ignored.",
+                    UserWarning,
+                )
+            self.generate_problem(X, y, preprocess_data=False)
+        else:
+            self._set_param_values()  # set parameter values
+
+        solver_options = self.solver_options if self.solver_options is not None else {}
+        if not isinstance(solver_options, dict):
+            raise TypeError("solver_options must be a dictionary")
+
+        self.coef_ = self._solve(X, y, solver_options, *args, **kwargs)
+        self._set_intercept(X_offset, y_offset, X_scale)
+
+        # return self for chaining fit and predict calls
+        return self
+
+    def _preprocess_data(
+        self, X: ArrayLike, y: ArrayLike, sample_weight: ArrayLike | None = None
+    ) -> tuple[ArrayLike, ArrayLike, ArrayLike, ArrayLike, ArrayLike]:
+        """Preprocess data for fitting."""
         if sample_weight is not None:
             sample_weight = _check_sample_weight(sample_weight, X, dtype=X.dtype)
             # rescale sample_weight to sum to number of samples
@@ -183,35 +224,7 @@ class CVXRegressor(RegressorMixin, LinearModel, metaclass=ABCMeta):
         if sample_weight is not None:
             X, y, _ = _rescale_data(X, y, sample_weight)
 
-        self._validate_params(X, y)
-
-        if not hasattr(self, "canonicals_") or self.warm_start is False:
-            self.generate_problem(X, y)
-
-        if self.warm_start is True:
-            # cache training data
-            if not hasattr(self, "cached_X_"):
-                self.cached_X_ = X
-            if not hasattr(self, "cached_y_"):
-                self.cached_y_ = y
-
-            # check if input data has changed and force reset accordingly
-            if not np.array_equal(self.cached_X_, X) or not np.array_equal(
-                self.cached_y_, y
-            ):
-                self.generate_problem(X, y)
-            else:
-                self._set_param_values()  # set parameter values
-
-        solver_options = self.solver_options if self.solver_options is not None else {}
-        if not isinstance(solver_options, dict):
-            raise TypeError("solver_options must be a dictionary")
-
-        self.coef_ = self._solve(X, y, solver_options, *args, **kwargs)
-        self._set_intercept(X_offset, y_offset, X_scale)
-
-        # return self for chaining fit and predict calls
-        return self
+        return X, y, X_offset, y_offset, X_scale
 
     def _validate_params(self, X: ArrayLike, y: ArrayLike) -> None:
         """Validate hyperparameter values.
@@ -233,6 +246,9 @@ class CVXRegressor(RegressorMixin, LinearModel, metaclass=ABCMeta):
 
     def _set_param_values(self) -> None:
         """Set the values of cvxpy parameters from param attributes for warm starts."""
+        if self._cvx_parameter_constraints is None:
+            return
+
         for parameter, value in self.get_params(deep=False).items():
             if parameter in self._cvx_parameter_constraints:
                 cvx_parameter = getattr(self.canonicals_.parameters, parameter)
@@ -274,8 +290,8 @@ class CVXRegressor(RegressorMixin, LinearModel, metaclass=ABCMeta):
                 for constraint in cvx_constraints[param_name]
             ]
 
-            # For now we will only set nonneg, nonpos, neg, pos, integer, boolean and/or shape
-            # of the cvxpy Parameter objects.
+            # For now we will only set nonneg, nonpos, neg, pos, integer, boolean and/or
+            # shape of the cvxpy Parameter objects.
             # TODO cxvpy only allows a single one of these to be set (except bool and integer)
             param_kwargs = {}
             for constraint in constraints:
@@ -322,7 +338,7 @@ class CVXRegressor(RegressorMixin, LinearModel, metaclass=ABCMeta):
     def _generate_auxiliaries(
         self, X: ArrayLike, y: ArrayLike, beta: cp.Variable, parameters: SimpleNamespace
     ) -> SimpleNamespace | None:
-        """Generate any auxiliary variables or expressions necessary in defining the objective.
+        """Generate any auxiliary variables/expressions necessary to define objective.
 
         Args:
             X (ArrayLike):
@@ -376,7 +392,7 @@ class CVXRegressor(RegressorMixin, LinearModel, metaclass=ABCMeta):
         beta: cp.Variable,
         parameters: SimpleNamespace | None = None,
         auxiliaries: SimpleNamespace | None = None,
-    ) -> list[cp.constraints] | None:
+    ) -> list[cp.constraints]:
         """Generate constraints for optimization problem.
 
         Args:
@@ -394,24 +410,47 @@ class CVXRegressor(RegressorMixin, LinearModel, metaclass=ABCMeta):
         Returns:
             list of cvxpy constraints
         """
-        return None
+        return []
 
-    def generate_problem(self, X: ArrayLike, y: ArrayLike) -> None:
+    def generate_problem(
+        self,
+        X: ArrayLike,
+        y: ArrayLike,
+        preprocess_data: bool = True,
+        sample_weight: ArrayLike | None = None,
+    ) -> None:
         """Generate regression problem and auxiliary cvxpy objects.
 
-        This initializes the minimization problem, the objective, coefficient variable (beta), problem parameters,
-        solution constraints, and auxiliary variables/terms.
+        This initializes the minimization problem, the objective, coefficient variable
+        (beta), problem parameters, solution constraints, and auxiliary variables/terms.
 
-        This is (almost always) called in the fit method, and not directly. However, it can be called directly if
-        further control over the problem is needed by accessing the canonicals_ objects. For example to add additional
-        constraints on problem variables.
+        This is (almost always) called in the fit method, and not directly. However, it
+        can be called directly if further control over the problem is needed by
+        accessing the canonicals_ objects. For example to add additional constraints on
+        problem variables.
 
         Args:
             X (ArrayLike):
                 Covariate/Feature matrix
             y (ArrayLike):
                 Target vector
+            preprocess_data (bool):
+                Whether to preprocess the data before generating the problem. If calling
+                generate_problem directly, this should be kept as True to ensure the
+                problem is generated correctly for a subsequent call to fit.
+            sample_weight (ArrayLike):
+                Individual weights for each sample of shape (n_samples,)
+                default=None. Only used if preprocess_data=True to rescale the data
+                accordingly.
         """
+        if preprocess_data is True:
+            X, y, _, _, _ = self._preprocess_data(X, y, sample_weight)
+
+        # X, y are cached to avoid re-generating problem if fit is called again with
+        # same data
+        self.cached_X_ = X
+        self.cached_y_ = y
+
         beta = cp.Variable(X.shape[1])
         parameters = self._generate_params(X, y)
         auxiliaries = self._generate_auxiliaries(X, y, beta, parameters)
@@ -425,6 +464,52 @@ class CVXRegressor(RegressorMixin, LinearModel, metaclass=ABCMeta):
             parameters=parameters,
             auxiliaries=auxiliaries,
             constraints=constraints,
+            user_constraints=[],
+        )
+
+    def add_constraints(
+        self, constraints: list[cp.constraint | cp.expressions]
+    ) -> None:
+        """Add a constraint to the problem.
+
+        .. warning::
+            Adding constraints will not work with any sklearn class that relies on
+            cloning the estimator (ie GridSearchCV, etc) . This is because a new cvxpy
+            problem is generated for any cloned estimator.
+
+        Args:
+            constraints (list of cp.constraint or cp.expressions):
+                cvxpy constraint to add to the problem
+        """
+        if not hasattr(self, "canonicals_"):
+            raise RuntimeError(
+                "Problem has not been generated. Please call generate_problem before"
+                " adding constraints."
+            )
+        self.canonicals_.user_constraints.extend(list(constraints))
+        # need to reset problem to update constraints
+        self._reset_problem()
+
+    def _reset_problem(self) -> None:
+        """Reset the cvxpy problem."""
+        if not hasattr(self, "canonicals_"):
+            raise RuntimeError(
+                "Problem has not been generated. Please call generate_problem before"
+                " resetting."
+            )
+
+        problem = cp.Problem(
+            cp.Minimize(self.canonicals_.objective),
+            self.canonicals_.constraints + self.canonicals_.user_constraints,
+        )
+        self.canonicals_ = CVXCanonicals(
+            problem=problem,
+            objective=self.canonicals_.objective,
+            beta=self.canonicals_.beta,
+            parameters=self.canonicals_.parameters,
+            auxiliaries=self.canonicals_.auxiliaries,
+            constraints=self.canonicals_.constraints,
+            user_constraints=self.canonicals_.user_constraints,
         )
 
     def _solve(
@@ -440,9 +525,9 @@ class CVXRegressor(RegressorMixin, LinearModel, metaclass=ABCMeta):
 class TikhonovMixin:
     """Mixin class to add a Tihhonov/ridge regularization term.
 
-    When using this Mixin, a cvxpy parameter named "eta" should be saved in the parameters
-    SimpleNamespace an attribute tikhonov_w can be added to allow a matrix otherwise simple l2/Ridge
-    is used.
+    When using this Mixin, a cvxpy parameter named "eta" should be saved in the
+    parameters SimpleNamespace an attribute tikhonov_w can be added to allow a matrix
+    otherwise simple l2/Ridge is used.
     """
 
     def _generate_objective(
