@@ -36,7 +36,7 @@ class BestSubsetSelection(MIQPl0):
             nonzero coefficients.
         big_M (float):
             Upper bound on the norm of coefficients associated with each
-            cluster (groups of coefficients) ||Beta_c||_2
+            groups of coefficients :math:`||\beta_c||_2`.
         hierarchy (list):
             A list of lists of integers storing hierarchy relations between
             coefficients.
@@ -152,7 +152,7 @@ class RidgedBestSubsetSelection(TikhonovMixin, BestSubsetSelection):
             L2 regularization hyper-parameter.
         big_M (float):
             Upper bound on the norm of coefficients associated with each
-            cluster (groups of coefficients) ||Beta_c||_2
+            groups of coefficients :math:`||\beta_c||_2`.
         hierarchy (list):
             A list of lists of integers storing hierarchy relations between
             coefficients.
